use crate::{
    http::error::Result,
    node::container::{kill, remove, restart, start, stop},
    AppState,
};
use axum::{
    extract::{Path, State},
    response::Redirect,
    routing::{get, post},
    Router,
};

mod log;
mod read;

macro_rules! container_command {
    ($cmd: ident) => {
        move |Path(cont_name): Path<String>, State(state): State<AppState>| async move {
            $cmd(&state.docker, &cont_name).await?;
            Ok(Redirect::to("./")) as Result<Redirect>
        }
    };
}

pub(super) fn router() -> Router<AppState> {
    Router::new()
        .route("/", get(read::get_all))
        .route(
            "/:container",
            get(|Path(cont_name): Path<String>| async move {
                Redirect::permanent(format!("{}/", &cont_name).as_str())
            }),
        )
        .route("/:container/", get(read::get_one))
        .route("/:container/logs", get(log::get))
        .route("/:container/start", post(container_command!(start)))
        .route("/:container/stop", post(container_command!(stop)))
        .route("/:container/restart", post(container_command!(restart)))
        .route("/:container/kill", post(container_command!(kill)))
        .route(
            "/:container/remove",
            post(
                move |Path(cont_name): Path<String>, State(state): State<AppState>| async move {
                    remove(&state.docker, &cont_name).await?;
                    Ok(Redirect::to("/")) as Result<Redirect>
                },
            ),
        )
}
